@@ -18,7 +18,7 @@
 
 import collections
 import copy
-from typing import Any, Dict, Sequence, Text, TypeVar
+from typing import Any, Dict, Optional, Sequence, Text, TypeVar
 
 import gin
 import numpy as np
@@ -130,6 +130,27 @@ def nested_lookup(nested_key: Text,
   return value
 
 
+def leaf_key(nested_key: Text,
+             delimiter: Text = '/') -> tf.Tensor:
+  """Returns the leaf node key name.
+
+  Args:
+    nested_key: String of the form "key/key/key...".
+    delimiter: String that splits the nested keys.
+
+  Returns:
+    value: Final leaf node key name.
+  """
+  # Parse the input string.
+  keys = nested_key.split(delimiter)
+  return keys[-1]
+
+
+def map_shape(x: Dict[Text, tf.Tensor]) -> Dict[Text, Sequence[int]]:
+  """Recursively infer tensor shapes for a dictionary of tensors."""
+  return tf.nest.map_structure(lambda t: list(tf.shape(t).numpy()), x)
+
+
 def pad_axis(x, padding=(0, 0), axis=0, **pad_kwargs):
   """Pads only one axis of a tensor.
 
@@ -849,8 +870,8 @@ def get_harmonic_frequencies(frequencies: tf.Tensor,
 
 def harmonic_synthesis(frequencies: tf.Tensor,
                        amplitudes: tf.Tensor,
-                       harmonic_shifts: tf.Tensor = None,
-                       harmonic_distribution: tf.Tensor = None,
+                       harmonic_shifts: Optional[tf.Tensor] = None,
+                       harmonic_distribution: Optional[tf.Tensor] = None,
                        n_samples: int = 64000,
                        sample_rate: int = 16000,
                        amp_resample_method: Text = 'window') -> tf.Tensor:
@@ -959,6 +980,27 @@ def linear_lookup(phase: tf.Tensor,
   return tf.reduce_sum(weighted_wavetables, axis=-1)
 
 
+def harmonic_distribution_to_wavetable(harmonic_distribution, n_wavetable=2048):
+  """Convert a harmonic distribution into a wavetable for synthesis.
+
+  Args:
+    harmonic_distribution: Shape [batch, time, n_harmonics], where the last axis
+      is normalized (sums to 1.0).
+    n_wavetable: Number of samples to have in the wavetable. If more than the
+      number of harmonics, performs interpolation/upsampling.
+
+  Returns:
+    A series of wavetables, shape [batch, time, n_wavetable]
+  """
+  n_harmonics = harmonic_distribution.shape[-1]
+  n_pad = int(n_wavetable/2 - n_harmonics)
+  # Pad the left for DC component, pad the right for wavetable interpolation.
+  fft_in = tf.pad(harmonic_distribution, [[0, 0], [0, 0], [1, n_pad]])
+  fft_in = tf.complex(fft_in, tf.zeros_like(fft_in))
+  wavetable = tf.signal.irfft(fft_in) * (n_wavetable / 2)
+  return wavetable
+
+
 def wavetable_synthesis(frequencies: tf.Tensor,
                         amplitudes: tf.Tensor,
                         wavetables: tf.Tensor,
@@ -1341,7 +1383,7 @@ def sinc_impulse_response(cutoff_frequency,
 
   if high_pass:
     # Invert filter.
-    pass_through = np.zeros_like(impulse_response)
+    pass_through = np.zeros(impulse_response.shape)
     pass_through[..., half_size] = 1.0
     pass_through = tf.convert_to_tensor(pass_through, dtype=tf.float32)
     impulse_response = pass_through - impulse_response
@@ -1382,7 +1424,7 @@ def frequency_filter(audio: tf.Tensor,
 def sinc_filter(audio: tf.Tensor,
                 cutoff_frequency: tf.Tensor,
                 window_size: int = 512,
-                sample_rate: int = None,
+                sample_rate: Optional[int] = None,
                 padding: Text = 'same',
                 high_pass: bool = False) -> tf.Tensor:
   """Filter audio with sinc low-pass filter.
